require 'spec_helper'

describe OO::Licence do
  let(:separator) { described_class.separator }
  let(:mask_mark) { described_class.mask_mark }

  describe '.separator' do
    let(:separator) { '.' }
    subject { described_class.separator }
    it { is_expected.to eq(separator) }
  end

  describe '.mask_mark' do
    let(:mask_mark) { '*' }
    subject { described_class.mask_mark }
    it { is_expected.to eq(mask_mark) }
  end

  describe '.max_depth' do
    let(:max_depth) { OO.configuration.licence_max_depth }
    subject { described_class.max_depth }

    context 'for default configuration' do
      it { is_expected.to eq(max_depth) }
    end
  end

  describe '.masks' do
    subject { described_class.masks }

    describe 'returns all possible top-level masks basing on max depth' do
      before(:each) { OO.configure { |c| c.licence_max_depth = max_depth } }

      context 'when max_depth is set to 1' do
        let(:max_depth) { 1 }
        let(:masks) { { '1' => mask_mark } }
        it { is_expected.to eq(masks) }
      end

      context 'when max_depth is set to 2' do
        let(:max_depth) { 2 }
        let(:masks) do
          { '1' => mask_mark,
            '2' => "#{mask_mark}#{separator}#{mask_mark}" }
        end

        it { is_expected.to eq(masks) }
      end

      context 'when max_depth is set to 3' do
        let(:max_depth) { 3 }
        let(:masks) do
          {
            '1' => mask_mark,
            '2' => "#{mask_mark}#{separator}#{mask_mark}",
            '3' => "#{mask_mark}#{separator}#{mask_mark}#{separator}#{mask_mark}"
          }
        end

        it { is_expected.to eq(masks) }
      end
    end
  end

  describe '#separator' do
    let(:licence) { build(:glass_take_with_juice) }
    subject { licence.separator }
    it { is_expected.to eq(described_class.separator) }
  end

  describe '#mask_mark' do
    let(:licence) { build(:glass_take_with_juice) }
    subject { licence.mask_mark }
    it { is_expected.to eq(described_class.mask_mark) }
  end

  describe '#key' do
    context 'for :glass_take_with_juice factory' do
      let(:licence) { build(:glass_take_with_juice) }

      subject { licence.key }

      it { is_expected.to be_a(String) }
      it { is_expected.to eq('glass.take.with_juice') }
    end
  end

  describe '#to_s' do
    context 'for :glass_take_with_juice factory' do
      let(:licence) { build(:glass_take_with_juice) }

      subject { licence.to_s }

      it { is_expected.to be_a(String) }
      it { is_expected.to eq('glass.take.with_juice') }
    end
  end

  describe '#to_a' do
    context 'for :glass_take_with_juice factory' do
      let(:licence) { build(:glass_take_with_juice) }

      subject { licence.to_a }

      it { is_expected.to be_an(Array) }
      it { is_expected.to eq(%w(glass take with_juice)) }
    end
  end

  describe '#depth' do
    context 'for :glass_take_with_juice factory' do
      let(:licence) { build(:glass_take_with_juice) }

      subject { licence.depth }

      it { is_expected.to be_a(Fixnum) }
      it { is_expected.to eq(3) }
    end
  end

  describe '#genre_masks' do
    let(:m) { mask_mark }
    let(:s) { separator }
    let(:genre_masks) do
      { depth_1: %W(#{m} #{m}#{s}#{m} #{m}#{s}#{m}#{s}#{m} #{m}#{s}#{m}#{s}#{m}#{s}#{m}),
        depth_3: %W(#{m}#{s}#{m}#{s}#{m} #{m}#{s}#{m}#{s}#{m}#{s}#{m}) }
    end

    subject { licence.genre_masks }

    describe 'should return the most general masks for licence' do
      context 'for :glass_take_with_juice factory (depth: 3)' do
        let(:licence) { build(:glass_take_with_juice) }

        it { is_expected.to be_an(Array) }
        it { is_expected.to eq(genre_masks[:depth_3]) }
      end

      context 'for :licence factory (depth: 1)' do
        let(:licence) { build(:licence) }

        it { is_expected.to be_an(Array) }
        it { is_expected.to eq(genre_masks[:depth_1]) }
      end
    end
  end

  describe '#mask_key(mask)' do
    let(:m) { mask_mark }
    let(:s) { separator }

    subject { licence.mask_key(mask) }

    describe 'for n-deep licence should replace mask\'s n chars with key\'s' do
      context 'when mask key is deeper than licence' do
        let(:mask) { "#{m}#{s}#{m}#{s}#{m}#{s}#{m}" }

        context 'for :glass_take_with_juice factory' do
          let(:licence) { build(:glass_take_with_juice) }

          it { is_expected.to be_a(String) }
          it { is_expected.to eq(licence.key + "#{s}#{m}") }
        end

        context 'for :licence factory' do
          let(:licence) { build(:licence) }

          it { is_expected.to be_an(String) }
          it { is_expected.to eq(licence.key + "#{s}#{m}#{s}#{m}#{s}#{m}") }
        end
      end

      context 'when mask depth equals licence depth' do
        context 'for :licence factory' do
          let(:licence) { build(:licence) }
          let(:mask) { "#{m}" }

          it { is_expected.to be_an(String) }
          it { is_expected.to eq(licence.key) }
        end
      end

      context 'when mask depth is less than licence depth' do
        context 'for :glass_take_with_juice factory' do
          let(:licence) { build(:glass_take_with_juice) }
          let(:mask) { "#{m}#{s}#{m}" }

          it { is_expected.to be_an(String) }
          it { is_expected.to eq(licence.key) }
        end
      end
    end
  end

  describe '#parent' do
    let(:mask_mark) { described_class.mask_mark }

    context 'for :glass_take_with_juice factory' do
      let(:licence) { build(:glass_take_with_juice) }
      let(:parent_licence) { licence.parent }

      subject { parent_licence }

      it { is_expected.to be_a(described_class) }

      describe '#key' do
        subject { parent_licence.key }
        it { is_expected.to eq('glass.take') }
      end
    end

    context 'for :licence factory' do
      let(:licence) { build(:licence) }
      let(:parent_licence) { licence.parent }

      describe '#key' do
        subject { parent_licence.key }
        it { is_expected.to eq(mask_mark) }
      end
    end

    context 'when key equals single mask mark' do
      let(:licence) { build(:licence, key: mask_mark) }
      let(:parent_licence) { licence.parent }

      describe '#key' do
        subject { parent_licence.key }
        it { is_expected.to eq(mask_mark) }
      end
    end
  end
end
